@@ -355,13 +355,18 @@ def fit_gmm(
 
 
 def threshold(
-    conn_map, percentile, subtract_mean=False, absolute_value=False, return_edges=False
+    conn_map,
+    percentile,
+    subtract_mean=False,
+    mean_weights=None,
+    absolute_value=False,
+    return_edges=False,
 ):
     """Return edges that exceed a threshold.
 
     Parameters
     ---------
-    conn_map : np.narray
+    conn_map : np.ndarray
         Connectivity matrix to threshold.
         Can be (n_components, n_modes, n_channels, n_channels),
         (n_modes, n_channels, n_channels) or (n_channels, n_channels).
@@ -372,6 +377,8 @@ def threshold(
         Should we subtract the mean over modes before thresholding?
         The thresholding is only done to identify edges, the values returned in
         conn_map are not mean subtracted.
+    mean_weights : np.ndarray
+        Weights when calculating the mean over modes.
     absolute_value : bool
         Should we take the absolute value before thresholding?
         The thresholding is only done to identify edges, the values returned in
@@ -416,17 +423,20 @@ def threshold(
     if n_modes == 1:
         subtract_mean = False
     if subtract_mean:
-        c -= np.mean(c, axis=1, keepdims=True)
+        c -= np.average(c, axis=1, weights=mean_weights, keepdims=True)
 
     # Take absolute value
     if absolute_value:
         c = abs(c)
 
+    # Set diagonal to nan
+    c[:, :, range(n_channels), range(n_channels)] = np.nan
+
     # Which edges are greater than the threshold?
     edges = np.empty([n_components, n_modes, n_channels, n_channels], dtype=bool)
     for i in range(n_components):
         for j in range(n_modes):
-            edges[i, j] = c[i, j] > np.percentile(c[i, j], percentile[i, j])
+            edges[i, j] = c[i, j] > np.nanpercentile(c[i, j], percentile[i, j])
 
     if return_edges:
         return np.squeeze(edges)
@@ -465,10 +475,15 @@ def save(
     parcellation_file,
     component=None,
     threshold=0,
+    glassbrain=False,
     plot_kwargs=None,
 ):
     """Save connectivity maps.
 
+    If glassbrain=True, this function is a wrapper for
+    nilearn.plotting.view_connectome, otherwise this function is a wrapper for
+    nilearn.plotting.plot_connectome.
+
     Parameters
     ----------
     connectivity_map : np.ndarray
@@ -484,10 +499,18 @@ def save(
         Threshold to determine which connectivity to show. Should be between 0 and 1.
         If a float is passed the same threshold is used for all modes. Otherwise,
         threshold should be a numpy array of shape (n_modes,).
+    glassbrain : bool
+        Sholud we create a 3D glass brain plot (as an interactive HTML file)
+        or a 2D image plot (as a png, pdf, svg, etc. file).
     plot_kwargs : dict
-        Keyword arguments to pass to nilearn.plotting.plot_connectome.
+        Keyword arguments to pass to the nilearn plotting function.
     """
     # Validation
+    if glassbrain and Path(filename).suffix != ".html":
+        raise ValueError(
+            "If glassbrain=True then filename must have a .html extension."
+        )
+
     error_message = (
         "Dimensionality of connectivity_map must be 3 or 4, "
         + f"got ndim={connectivity_map.ndim}."
@@ -512,7 +535,7 @@ def save(
     parcellation = Parcellation(parcellation_file)
 
     # Select the component we're plotting
-    conn_map = connectivity_map[component]
+    conn_map = np.copy(connectivity_map[component])
 
     # Default plotting settings
     default_plot_kwargs = {"node_size": 10, "node_color": "black"}
@@ -524,19 +547,40 @@ def save(
         # Overwrite keyword arguments if passed
         kwargs = override_dict_defaults(default_plot_kwargs, plot_kwargs)
 
-        # If all connections are zero don't add a colourbar
-        kwargs["colorbar"] = np.any(
-            conn_map[i][~np.eye(conn_map[i].shape[-1], dtype=bool)] != 0
-        )
-
-        # Plot maps
+        # Output filename
         output_file = "{fn.parent}/{fn.stem}{i:0{w}d}{fn.suffix}".format(
             fn=Path(filename), i=i, w=len(str(n_modes))
         )
-        plotting.plot_connectome(
-            conn_map[i],
-            parcellation.roi_centers(),
-            edge_threshold=f"{threshold[i] * 100}%",
-            output_file=output_file,
-            **kwargs,
-        )
+
+        if glassbrain:
+            # The colour bar range is determined by the max value in the matrix
+            # we zero the diagonal so it's not included
+            np.fill_diagonal(conn_map[i], val=0)
+
+            # Plot thick lines for the connections
+            if "linewidth" not in kwargs:
+                kwargs["linewidth"] = 12
+
+            # Plot maps
+            connectome = plotting.view_connectome(
+                conn_map[i],
+                parcellation.roi_centers(),
+                edge_threshold=f"{threshold[i] * 100}%",
+                **kwargs,
+            )
+            connectome.save_as_html(output_file)
+
+        else:
+            # If all connections are zero don't add a colourbar
+            kwargs["colorbar"] = np.any(
+                conn_map[i][~np.eye(conn_map[i].shape[-1], dtype=bool)] != 0
+            )
+
+            # Plot maps
+            plotting.plot_connectome(
+                conn_map[i],
+                parcellation.roi_centers(),
+                edge_threshold=f"{threshold[i] * 100}%",
+                output_file=output_file,
+                **kwargs,
+            )
