@@ -6,7 +6,6 @@ import numpy as np
 from scipy import stats, special
 from sklearn.mixture import BayesianGaussianMixture, GaussianMixture
 
-from osl_dynamics.data import processing
 from osl_dynamics.utils import plotting
 
 
@@ -79,10 +78,12 @@ def fit_gaussian_mixture(
     # Logit transform
     if logit_transform:
         X = special.logit(X)
+        X[np.isinf(X[:, 0]), :] = np.mean(X[~np.isinf(X[:, 0]), 0])
 
     # Standardise the data
     if standardize:
-        X = processing.standardize(X, create_copy=False)
+        X -= np.mean(X, axis=0)
+        X /= np.std(X, axis=0)
 
     # Fit a Gaussian mixture model
     if bayesian:
