@@ -9,6 +9,7 @@ import numpy as np
 from pathlib import Path
 from nilearn import plotting
 from tqdm import trange
+
 from osl_dynamics import array_ops, files
 from osl_dynamics.analysis.spectral import get_frequency_args_range
 
@@ -152,6 +153,13 @@ def power_map_grid(mask_file, parcellation_file, power_map):
 
     # Make a 2D array of voxel weights for each parcel
     n_parcels = parcellation.shape[-1]
+
+    # check parcellation is compatible:
+    if power_map.shape[1] is not n_parcels:
+        print(
+            "Error: parcellation_file has a different number of parcels to the power_maps"
+        )
+
     voxel_weights = parcellation_grid.reshape(-1, n_parcels, order="F")[non_zero_voxels]
 
     # Normalise the voxels weights
@@ -161,6 +169,7 @@ def power_map_grid(mask_file, parcellation_file, power_map):
     n_voxels = voxel_weights.shape[0]
     n_modes = power_map.shape[0]
     spatial_map_values = np.empty([n_voxels, n_modes])
+
     for i in range(n_modes):
         spatial_map_values[:, i] = voxel_weights @ power_map[i]
 
@@ -195,7 +204,7 @@ def save(
         is equal to n_channels.
     filename : str
         Output filename. If extension is .nii.gz the power map is saved as a
-        NIFTI file. Or if the extension is png, it is saved as images.
+        NIFTI file. Or if the extension is png/svg/pdf, it is saved as images.
     mask_file : str
         Mask file used to preprocess the training data.
     parcellation_file : str
@@ -214,8 +223,12 @@ def save(
     power_map = np.copy(power_map)
 
     # Validation
-    if ".nii.gz" not in filename and ".png" not in filename:
-        raise ValueError("filename must have extension .nii.gz or .png.")
+    allowed_extensions = [".nii.gz", ".png", ".svg", ".pdf"]
+    if not any([ext in filename for ext in allowed_extensions]):
+        raise ValueError(
+            "filename must have one of following extensions: "
+            + f"{' '.join(allowed_extensions)}."
+        )
     mask_file = files.check_exists(mask_file, files.mask.directory)
     parcellation_file = files.check_exists(
         parcellation_file, files.parcellation.directory
@@ -230,7 +243,9 @@ def save(
             # A n_channels by n_channels array has been passed,
             # extract the diagonal (the np.copy is needed because np.diagonal
             # returns a read-only array (this started in NumPy 1.9))
-            power_map = np.copy(np.diagonal(power_map, axis1=-2, axis2=-1))
+            power_map = np.copy(np.diagonal(power_map, axis1=-2, axis2=-1))[
+                np.newaxis, ...
+            ]
     else:
         power_map = power_map[np.newaxis, ...]
 
@@ -266,7 +281,7 @@ def save(
         nib.save(nii, filename)
 
     # Save each map as an image
-    elif ".png" in filename:
+    else:
         n_modes = power_map.shape[-1]
         for i in trange(n_modes, desc="Saving images", ncols=98):
             nii = nib.Nifti1Image(power_map[:, :, :, i], mask.affine, mask.header)
