@@ -3,13 +3,14 @@
 """
 
 import warnings
+
 import numpy as np
 from scipy.signal.windows import dpss, hann
 from sklearn.decomposition import non_negative_factorization
 from tqdm import trange
+
 from osl_dynamics import array_ops
 from osl_dynamics.analysis import regression, time_series
-from osl_dynamics.data import processing
 
 
 def nextpow2(x):
@@ -530,7 +531,7 @@ def multitaper_spectra(
 
     # Standardise before calculating the multitaper
     if standardize:
-        data = [processing.standardize(d) for d in data]
+        data = [(d - np.mean(d, axis=0)) / np.std(d, axis=0) for d in data]
 
     print("Calculating power spectra")
     power_spectra = []
@@ -572,6 +573,14 @@ def multitaper_spectra(
                     time_series_segment = np.pad(time_series_segment, n_padding)[
                         :segment_length, n_padding:-n_padding
                     ]
+                    if time_series_segment.shape[0] == segment_length - 1:
+                        time_series_segment = np.append(
+                            time_series_segment,
+                            np.zeros(
+                                [1, time_series_segment.shape[1]], dtype=np.float32
+                            ),
+                            axis=0,
+                        )
 
                 # Calculate the power (and cross) spectrum using the multitaper method
                 p[i] += multitaper(
@@ -698,7 +707,7 @@ def regression_spectra(
 
     # Standardise before calculating the spectrogram
     if standardize:
-        data = [processing.standardize(d) for d in data]
+        data = [(d - np.mean(d, axis=0)) / np.std(d, axis=0) for d in data]
 
     # Remove data points not in alpha due to time embedding the training data
     if n_embeddings is not None:
