@@ -7,6 +7,7 @@ from pathlib import Path
 from scipy.optimize import linear_sum_assignment
 from osl_dynamics import array_ops, analysis
 from osl_dynamics.inference import metrics
+from osl_dynamics.utils.misc import override_dict_defaults
 
 
 def argmax_time_courses(
@@ -418,75 +419,76 @@ def fano_factor(
     return np.squeeze(F)
 
 
-def gmm_per_subject(time_course, time_course_type, gmm_filename=None):
-    """Fit a 2 component GMM to a subject specific time course.
+def fit_gmm(
+    time_course,
+    logit_transform=True,
+    standardize=True,
+    gmm_filename=None,
+    sklearn_kwargs=None,
+):
+    """Fit a two component GMM on the mode time courses.
 
     Parameters
     ----------
-    time_course : np.ndarray
-        Time course to be fitted with GMM. Shape is (n_samples, n_modes)
-    time_course_type : str
-        Name of the time course. (e.g. alpha/gamma)
+    time_course : List of np.ndarray or np.ndarray
+        Mode time courses.
+    logit_transform : bool
+        Should we logit transform the mode time course?
+    standardize : bool
+        Should we standardize the mode time course?
     gmm_filename : str
-        Path to directory to store the GMM fit plots.
+        Path to directory to plot the GMM fit plots.
+    sklearn_kwargs : dict
+        keyword arguments for sklearn's GaussianMixture.
 
     Returns
     -------
-    time_course : np.ndarray
-        Time course in which activations are determined by fitting GMM.
+    gmm_time_course : List of np.ndarray or np.ndarray
+        GMM fitted mode time courses with binary entries.
     """
+
+    if isinstance(time_course, list):
+        # Extract positions of discontinuities
+        discontinuities = [mtc.shape[0] for mtc in time_course]
+        time_course = np.concatenate(time_course)
+    else:
+        discontinuities = None
+
     n_modes = time_course.shape[1]
 
-    # loop over modes
-    for j in range(n_modes):
-        a = time_course[:, j]
-        a[np.isinf(a)] = np.mean(a[~np.isinf(a)])
+    # Initialise an array to hold the gmm thresholded time course
+    gmm_time_course = np.empty(time_course.shape)
 
-        # fit a 2 component GMM
+    # Loop over modes
+    for mode in range(n_modes):
+        a = time_course[:, mode]
+
+        # GMM plot filename
         if gmm_filename is not None:
-            plot_filename = "{fn.parent}/{fn.stem}/{fn.stem}_{time_course_type}_{j:0{w2}d}{fn.suffix}".format(
+            plot_filename = "{fn.parent}/{fn.stem}{mode:0{w}d}{fn.suffix}".format(
                 fn=Path(gmm_filename),
-                time_course_type=time_course_type,
-                j=j,
-                w2=len(str(n_modes)),
+                mode=mode,
+                w=len(str(n_modes)),
             )
         else:
             plot_filename = None
+
+        # Fit the GMM
+        default_sklearn_kwargs = {"max_iter": 5000, "n_init": 5}
+        sklearn_kwargs = override_dict_defaults(default_sklearn_kwargs, sklearn_kwargs)
         mixture_label = analysis.gmm.fit_gaussian_mixture(
             a,
-            print_message=False,
-            plot_filename=plot_filename,
             bayesian=False,
-            max_iter=5000,
-            n_init=5,
+            logit_transform=logit_transform,
+            standardize=standardize,
+            sklearn_kwargs=sklearn_kwargs,
+            plot_filename=plot_filename,
+            print_message=False,
+            return_labels=True,
         )
-        time_course[:, j] = np.array(mixture_label)
-    return time_course
+        gmm_time_course[:, mode] = np.array(mixture_label)
 
+    if discontinuities is None:
+        return gmm_time_course
 
-def gmm_for_time_course(time_course, time_course_type, gmm_filename=None):
-    """Fit GMM to time course
-
-    Parameters
-    ----------
-    time_course : List of np.ndarray
-        List of time courses per subject.
-    time_course_type : str
-        Name of the time course (e.g. alpha/gamma).
-    gmm_filename : str
-        Path to directory to store the GMM fit plots.
-
-    Returns
-    -------
-    time_courses : list of np.ndarray
-        List of time courses in which activations are determined by fitting GMM.
-    """
-    # extract the positions of discontinuities
-    discontinuities = [mtc.shape[0] for mtc in time_course]
-
-    # gmm fit to the concatenated time course across subjects
-    time_course = gmm_per_subject(
-        np.concatenate(time_course), time_course_type, gmm_filename=gmm_filename
-    )
-
-    return np.split(time_course, np.cumsum(discontinuities))
+    return np.split(gmm_time_course, np.cumsum(discontinuities))
