@@ -39,6 +39,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
 
     Parameters
     ----------
+    model_name : str
+        Model name.
     n_modes : int
         Number of modes.
     n_channels : int
@@ -59,6 +61,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         E.g. 'relu', 'elu', etc.
     inference_dropout : float
         Dropout rate.
+    inference_regularizer : str
+        Regularizer.
 
     model_rnn : str
         RNN to use, either 'gru' or 'lstm'.
@@ -73,6 +77,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         E.g. 'relu', 'elu', etc.
     model_dropout : float
         Dropout rate.
+    model_regularizer : str
+        Regularizer.
 
     theta_normalization : str
         Type of normalization to apply to the posterior samples, theta.
@@ -122,6 +128,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         Strategy for distributed learning.
     """
 
+    model_name: str = "DyNeMo"
+
     # Inference network parameters
     inference_rnn: Literal["gru", "lstm"] = "lstm"
     inference_n_layers: int = 1
@@ -129,6 +137,7 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
     inference_normalization: Literal[None, "batch", "layer"] = None
     inference_activation: str = None
     inference_dropout: float = 0.0
+    inference_regularizer: str = None
 
     # Model network parameters
     model_rnn: Literal["gru", "lstm"] = "lstm"
@@ -137,6 +146,7 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
     model_normalization: Literal[None, "batch", "layer"] = None
     model_activation: str = None
     model_dropout: float = 0.0
+    model_regularizer: str = None
 
     # Observation model parameters
     learn_means: bool = None
@@ -174,6 +184,8 @@ class Model(VariationalInferenceModelBase):
     config : osl_dynamics.models.dynemo.Config
     """
 
+    config_type = Config
+
     def build_model(self):
         """Builds a keras model."""
         self.model = _model_structure(self.config)
@@ -200,6 +212,10 @@ class Model(VariationalInferenceModelBase):
         """
         return dynemo_obs.get_means_covariances(self.model)
 
+    def get_observation_model_parameters(self):
+        """Wrapper for get_means_covariances."""
+        return self.get_means_covariances()
+
     def set_means(self, means, update_initializer=True):
         """Set the means of each mode.
 
@@ -226,6 +242,19 @@ class Model(VariationalInferenceModelBase):
         """
         dynemo_obs.set_covariances(self.model, covariances, update_initializer)
 
+    def set_observation_model_parameters(
+        self, observation_model_parameters, update_initializer=True
+    ):
+        """Wrapper for set_means and set_covariances."""
+        self.set_means(
+            observation_model_parameters[0],
+            update_initializer=update_initializer,
+        )
+        self.set_covariances(
+            observation_model_parameters[1],
+            update_initializer=update_initializer,
+        )
+
     def set_regularizers(self, training_dataset):
         """Set the means and covariances regularizer based on the training data.
 
@@ -246,93 +275,6 @@ class Model(VariationalInferenceModelBase):
         if self.config.learn_covariances:
             dynemo_obs.set_covariances_regularizer(self.model, training_dataset)
 
-    def random_subject_initialization(
-        self, training_data, n_epochs, n_subjects, n_kl_annealing_epochs=None, **kwargs
-    ):
-        """Initialization for the mode means/covariances.
-
-        Pick a subject at random, train a model, repeat a few times. Use
-        the means/covariances from the best model (judged using the final loss).
-
-        Parameters
-        ----------
-        training_data : list of tensorflow.data.Dataset or osl_dynamics.data.Data
-            Datasets for each subject.
-        n_epochs : int
-            Number of epochs to train.
-        n_subjects : int
-            How many subjects should we train on?
-        n_kl_annealing_epochs : int
-            Number of KL annealing epochs to use during initialization. If None
-            then the KL annealing epochs in the config is used.
-        kwargs : keyword arguments
-            Keyword arguments for the fit method.
-        """
-        print("Random subject initialization for mode means and covariances:")
-
-        # Original number of KL annealing epochs
-        original_n_kl_annealing_epochs = self.config.n_kl_annealing_epochs
-
-        # Use n_kl_annealing_epochs if passed
-        self.config.n_kl_annealing_epochs = (
-            n_kl_annealing_epochs or original_n_kl_annealing_epochs
-        )
-
-        # Make a list of tensorflow Datasets if the data
-        training_data = self.make_dataset(
-            training_data, shuffle=True, concatenate=False
-        )
-
-        if not isinstance(training_data, list):
-            raise ValueError(
-                "training_data must be a list of Datasets or a Data object."
-            )
-
-        # Pick n_subjects at random
-        n_all_subjects = len(training_data)
-        subjects_to_use = np.random.choice(
-            range(n_all_subjects), n_subjects, replace=False
-        )
-
-        # Train the model a few times and keep the best one
-        best_loss = np.Inf
-        losses = []
-        for subject in subjects_to_use:
-            print("Using subject", subject)
-
-            # Get the dataset for this subject
-            subject_dataset = training_data[subject]
-
-            # Reset the model weights and train
-            self.reset()
-            history = self.fit(subject_dataset, epochs=n_epochs, **kwargs)
-            loss = history.history["loss"][-1]
-            losses.append(loss)
-            print(f"Subject {subject} loss: {loss}")
-
-            # Record the loss of this subject's data
-            if loss < best_loss:
-                best_loss = loss
-                subject_chosen = subject
-                best_weights = self.get_weights()
-
-        print(f"Using means and covariances from subject {subject_chosen}")
-
-        # Restore the best model and get the inferred means/covariances
-        # for initialisation
-        self.set_weights(best_weights)
-        init_means, init_covs = self.get_means_covariances()
-
-        # Reset model for full training
-        self.reset()
-
-        # Set initial means/covariances
-        self.set_means(init_means, update_initializer=True)
-        self.set_covariances(init_covs, update_initializer=True)
-
-        # Reset the number of KL annealing epochs
-        self.config.n_kl_annealing_epochs = original_n_kl_annealing_epochs
-
     def sample_alpha(self, n_samples, theta_norm=None):
         """Uses the model RNN to sample mode mixing factors, alpha.
 
@@ -421,6 +363,7 @@ def _model_structure(config):
         config.inference_n_layers,
         config.inference_n_units,
         config.inference_dropout,
+        config.inference_regularizer,
         name="inf_rnn",
     )
     inf_mu_layer = layers.Dense(config.n_modes, name="inf_mu")
@@ -492,6 +435,7 @@ def _model_structure(config):
         config.model_n_layers,
         config.model_n_units,
         config.model_dropout,
+        config.model_regularizer,
         name="mod_rnn",
     )
     mod_mu_layer = layers.Dense(config.n_modes, name="mod_mu")
