@@ -7,7 +7,7 @@ NOTE:
 
 """
 
-import os
+import os.path as op
 from dataclasses import dataclass
 
 import numpy as np
@@ -15,6 +15,7 @@ import tensorflow as tf
 import tensorflow_probability as tfp
 from tensorflow.keras import layers, utils
 
+import osl_dynamics.data.tf as dtf
 from osl_dynamics.simulation import HMM
 from osl_dynamics.models import dynemo_obs
 from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
@@ -31,7 +32,7 @@ from numpy.ctypeslib import ndpointer
 # This library has has already been compiled for BMRC.
 # Please re-compile if running on a different computer with:
 # `python setup.py build`, and update the path.
-libfile = "/well/woolrich/projects/software/hmm_inference_libc/build/lib.linux-x86_64-3.8/hidden_state_inference.so"
+libfile = "/well/woolrich/projects/software/hmm_inference_libc/build/lib.linux-x86_64-cpython-310/hidden_state_inference.so"
 hidden_state_inference = CDLL(libfile)
 
 
@@ -41,6 +42,8 @@ class Config(BaseModelConfig):
 
     Parameters
     ----------
+    model_name : str
+        Model name.
     n_states : int
         Number of states.
     n_channels : int
@@ -68,13 +71,15 @@ class Config(BaseModelConfig):
     n_epochs : int
         Number of training epochs.
     optimizer : str or tensorflow.keras.optimizers.Optimizer
-        Optimizer to use. 'adam' is recommended.
+        Optimizer to use.
     multi_gpu : bool
         Should be use multiple GPUs for training?
     strategy : str
         Strategy for distributed learning.
     """
 
+    model_name: str = "HMM"
+
     # Observation model parameters
     learn_means: bool = None
     learn_covariances: bool = None
@@ -106,6 +111,8 @@ class Model(ModelBase):
     config : osl_dynamics.models.hmm.Config
     """
 
+    config_type = Config
+
     def build_model(self):
         """Builds a keras model."""
         self.model = _model_structure(self.config)
@@ -183,6 +190,70 @@ class Model(ModelBase):
 
         return history
 
+    def random_subset_initialization(
+        self, training_data, n_epochs, n_init, take, **kwargs
+    ):
+        """Random subset initialization.
+
+        The model is trained for a few epochs with different random subsets
+        of the training dataset. The model with the best free energy is kept.
+
+        Parameters
+        ----------
+        training_data : tensorflow.data.Dataset or osl_dynamics.data.Data
+            Dataset to use for training.
+        n_epochs : int
+            Number of epochs to train the model.
+        n_init : int
+            Number of initializations.
+        take : float
+            Fraction of total batches to take.
+        kwargs : keyword arguments
+            Keyword arguments for the fit method.
+
+        Returns
+        -------
+        history : history
+            The training history of the best initialization.
+        """
+        if n_init is None or n_init == 0:
+            print(
+                "Number of initializations was set to zero. "
+                + "Skipping initialization."
+            )
+            return
+
+        print("Random subset initialization:")
+
+        # Make a TensorFlow Dataset
+        training_data = self.make_dataset(training_data, shuffle=True, concatenate=True)
+
+        # Calculate the number of batches to use
+        n_total_batches = dtf.get_n_batches(training_data)
+        n_batches = max(round(n_total_batches * take), 1)
+        print(f"Using {n_batches} out of {n_total_batches} batches")
+
+        # Pick the initialization with the lowest free energy
+        best_loss = np.Inf
+        for n in range(n_init):
+            print(f"Initialization {n}")
+            self.reset()
+            training_data.shuffle(100000)
+            training_data_subset = training_data.take(n_batches)
+            history = self.fit(training_data_subset, epochs=n_epochs, **kwargs)
+            loss = history["loss"][-1]
+            if loss < best_loss:
+                best_initialization = n
+                best_loss = loss
+                best_history = history
+                best_weights = self.get_weights()
+
+        print(f"Using initialization {best_initialization}")
+        self.reset()
+        self.set_weights(best_weights)
+
+        return best_history
+
     def _get_state_probs(self, x):
         """Get state probabilities.
 
@@ -415,7 +486,7 @@ class Model(ModelBase):
 
         A multivariate normal prior is applied to the mean vectors with mu = 0,
         sigma=diag((range / 2)**2) and an inverse Wishart prior is applied to the
-        covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(1 / range).
+        covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(range).
 
         Parameters
         ----------
@@ -482,7 +553,7 @@ class Model(ModelBase):
             self.config.sequence_length, prepared=prepared, concatenate=concatenate
         )
 
-    def save_all(self, dirname):
+    def save_weights(self, dirname):
         """Save all model weights.
 
         Parameters
@@ -490,11 +561,10 @@ class Model(ModelBase):
         dirname : str
             Location to save model parameters to.
         """
-        os.system("mkdir {}".format(dirname))
-        self.save_weights(os.path.join(dirname, "model_weights"))
-        np.save(os.path.join(dirname, "model_trans_prob.npy"), self.trans_prob)
+        self.model.save_weights(op.join(dirname, "weights"))
+        np.save(op.join(dirname, "trans_prob.npy"), self.trans_prob)
 
-    def load_all(self, dirname):
+    def load_weights(self, dirname):
         """Load all model parameters.
 
         Parameters
@@ -502,8 +572,8 @@ class Model(ModelBase):
         dirname : str
             Location to load model parameters from.
         """
-        self.load_weights(os.path.join(dirname, "model_weights"))
-        self.trans_prob = np.load(os.path.join(dirname, "model_trans_prob.npy"))
+        self.model.load_weights(op.join(dirname, "weights"))
+        self.trans_prob = np.load(op.join(dirname, "trans_prob.npy"))
 
 
 def _model_structure(config):
