@@ -7,6 +7,7 @@ from typing import Literal
 
 import numpy as np
 
+import osl_dynamics.data.tf as dtf
 from osl_dynamics.models.mod_base import ModelBase
 from osl_dynamics.inference import callbacks, initializers
 from osl_dynamics.utils.misc import replace_argument
@@ -115,17 +116,19 @@ class VariationalInferenceModelBase(ModelBase):
 
         return super().fit(*args, **kwargs)
 
-    def multistart_initialization(
+    def random_subset_initialization(
         self,
         training_data,
         n_epochs,
         n_init,
+        take,
+        n_kl_annealing_epochs=None,
         **kwargs,
     ):
-        """Multi-start initialization.
+        """Random subset initialization.
 
-        The model is trained for a few epochs with different random initializations
-        for weights and the model with the best free energy is kept.
+        The model is trained for a few epochs with different random subsets
+        of the training dataset. The model with the best free energy is kept.
 
         Parameters
         ----------
@@ -135,6 +138,12 @@ class VariationalInferenceModelBase(ModelBase):
             Number of epochs to train the model.
         n_init : int
             Number of initializations.
+        take : float
+            Fraction of total batches to take.
+        n_kl_annealing_epochs : int
+            Number of KL annealing epochs.
+        kwargs : keyword arguments
+            Keyword arguments for the fit method.
 
         Returns
         -------
@@ -148,36 +157,169 @@ class VariationalInferenceModelBase(ModelBase):
             )
             return
 
+        print("Random subset initialization:")
+
+        # Original number of KL annealing epochs
+        original_n_kl_annealing_epochs = self.config.n_kl_annealing_epochs
+
+        # Use n_kl_annealing_epochs if passed
+        self.config.n_kl_annealing_epochs = (
+            n_kl_annealing_epochs or original_n_kl_annealing_epochs
+        )
+
+        # Make a TensorFlow Dataset
+        training_data = self.make_dataset(training_data, shuffle=True, concatenate=True)
+
+        # Calculate the number of batches to use
+        n_total_batches = dtf.get_n_batches(training_data)
+        n_batches = max(round(n_total_batches * take), 1)
+        print(f"Using {n_batches} out of {n_total_batches} batches")
+
         # Pick the initialization with the lowest free energy
-        best_free_energy = np.Inf
+        best_loss = np.Inf
         for n in range(n_init):
             print(f"Initialization {n}")
             self.reset()
-            history = self.fit(
-                training_data,
-                epochs=n_epochs,
-                **kwargs,
-            )
-            free_energy = history.history["loss"][-1]
-            if free_energy < best_free_energy:
+            training_data.shuffle(100000)
+            training_data_subset = training_data.take(n_batches)
+            history = self.fit(training_data_subset, epochs=n_epochs, **kwargs)
+            loss = history.history["loss"][-1]
+            if loss < best_loss:
                 best_initialization = n
-                best_free_energy = free_energy
-                best_weights = self.model.get_weights()
+                best_loss = loss
                 best_history = history
+                best_weights = self.get_weights()
 
         print(f"Using initialization {best_initialization}")
         self.reset()
         self.set_weights(best_weights)
+        self.reset_kl_annealing_factor()
+
+        # Reset the number of KL annealing epochs
+        self.config.n_kl_annealing_epochs = original_n_kl_annealing_epochs
 
         return best_history
 
+    def single_subject_initialization(
+        self, training_data, n_epochs, n_init, n_kl_annealing_epochs=None, **kwargs
+    ):
+        """Initialization for the mode means/covariances.
+
+        Pick a subject at random, train a model, repeat a few times. Use
+        the means/covariances from the best model (judged using the final loss).
+
+        Parameters
+        ----------
+        training_data : list of tensorflow.data.Dataset or osl_dynamics.data.Data
+            Datasets for each subject.
+        n_epochs : int
+            Number of epochs to train.
+        n_init : int
+            How many subjects should we train on?
+        n_kl_annealing_epochs : int
+            Number of KL annealing epochs to use during initialization. If None
+            then the KL annealing epochs in the config is used.
+        kwargs : keyword arguments
+            Keyword arguments for the fit method.
+        """
+        if n_init is None or n_init == 0:
+            print(
+                "Number of initializations was set to zero. "
+                + "Skipping initialization."
+            )
+            return
+
+        print("Single subject initialization:")
+
+        # Original number of KL annealing epochs
+        original_n_kl_annealing_epochs = self.config.n_kl_annealing_epochs
+
+        # Use n_kl_annealing_epochs if passed
+        self.config.n_kl_annealing_epochs = (
+            n_kl_annealing_epochs or original_n_kl_annealing_epochs
+        )
+
+        # Make a list of tensorflow Datasets if the data
+        training_data = self.make_dataset(
+            training_data, shuffle=True, concatenate=False
+        )
+
+        if not isinstance(training_data, list):
+            raise ValueError(
+                "training_data must be a list of Datasets or a Data object."
+            )
+
+        # Pick n_init subjects at random
+        n_all_subjects = len(training_data)
+        subjects_to_use = np.random.choice(range(n_all_subjects), n_init, replace=False)
+
+        # Train the model a few times and keep the best one
+        best_loss = np.Inf
+        losses = []
+        for subject in subjects_to_use:
+            print("Using subject", subject)
+
+            # Get the dataset for this subject
+            subject_dataset = training_data[subject]
+
+            # Reset the model weights and train
+            self.reset()
+            history = self.fit(subject_dataset, epochs=n_epochs, **kwargs)
+            loss = history.history["loss"][-1]
+            losses.append(loss)
+            print(f"Subject {subject} loss: {loss}")
+
+            # Record the loss of this subject's data
+            if loss < best_loss:
+                best_loss = loss
+                subject_chosen = subject
+                best_weights = self.get_weights()
+
+        print(f"Using means and covariances from subject {subject_chosen}")
+
+        # Use the weights from the best initialisation for the full training
+        self.reset()
+        self.set_weights(best_weights)
+        self.reset_kl_annealing_factor()
+
+        # Reset the number of KL annealing epochs
+        self.config.n_kl_annealing_epochs = original_n_kl_annealing_epochs
+
+    def multistart_initialization(
+        self,
+        training_data,
+        n_epochs,
+        n_init,
+        n_kl_annealing_epochs=None,
+        **kwargs,
+    ):
+        """Multi-start initialization.
+
+        Wrapper for random_subset_initialization with take=1.
+
+        Returns
+        -------
+        history : history
+            The training history of the best initialization.
+        """
+
+        return self.random_subset_initialization(
+            training_data,
+            n_epochs,
+            n_init,
+            take=1,
+            n_kl_annealing_epochs=n_kl_annealing_epochs,
+            **kwargs,
+        )
+
     def reset_kl_annealing_factor(self):
         """Sets the KL annealing factor to zero.
 
         This method assumes there is a keras layer named 'kl_loss' in the model.
         """
-        kl_loss_layer = self.model.get_layer("kl_loss")
-        kl_loss_layer.annealing_factor.assign(0.0)
+        if self.config.do_kl_annealing:
+            kl_loss_layer = self.model.get_layer("kl_loss")
+            kl_loss_layer.annealing_factor.assign(0.0)
 
     def reset_weights(self, keep=None):
         """Reset the model as if you've built a new model.
@@ -188,10 +330,9 @@ class VariationalInferenceModelBase(ModelBase):
             Layer names to NOT reset.
         """
         initializers.reinitialize_model_weights(self.model, keep)
-        if self.config.do_kl_annealing:
-            self.reset_kl_annealing_factor()
+        self.reset_kl_annealing_factor()
 
-    def predict(self, *args, **kwargs) -> dict:
+    def predict(self, *args, **kwargs):
         """Wrapper for the standard keras predict method.
 
         Returns
@@ -224,9 +365,7 @@ class VariationalInferenceModelBase(ModelBase):
             (n_samples, n_modes).
         """
         if self.config.multiple_dynamics:
-            return self.get_mode_time_courses(
-                inputs, *args, concatenate=concatenate, **kwargs
-            )
+            return self.get_mode_time_courses(inputs, concatenate=concatenate)
 
         inputs = self.make_dataset(inputs, concatenate=concatenate)
 
