@@ -16,7 +16,6 @@ from osl_dynamics.models.inf_mod_base import (
     VariationalInferenceModelConfig,
     VariationalInferenceModelBase,
 )
-from osl_dynamics.inference import regularizers
 from osl_dynamics.inference.layers import (
     InferenceRNNLayer,
     LogLikelihoodLossLayer,
@@ -42,8 +41,12 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
 
     Parameters
     ----------
+    model_name : str
+        Model name.
     n_modes : int
-        Number of modes.
+        Number of modes for both power.
+    n_fc_modes : int
+        Number of modes for FC. If None, then set to n_modes.
     n_channels : int
         Number of channels.
     sequence_length : int
@@ -62,6 +65,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         E.g. 'relu', 'elu', etc.
     inference_dropout : float
         Dropout rate.
+    inference_regularizer : str
+        Regularizer.
 
     model_rnn : str
         RNN to use, either 'gru' or 'lstm'.
@@ -76,6 +81,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         E.g. 'relu', 'elu', etc.
     model_dropout : float
         Dropout rate.
+    model_regularizer : str
+        Regularizer.
 
     theta_normalization : str
         Type of normalization to apply to the posterior samples, theta.
@@ -134,6 +141,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         Strategy for distributed learning.
     """
 
+    model_name: str = "M-DyNeMo"
+
     # Inference network parameters
     inference_rnn: Literal["gru", "lstm"] = "lstm"
     inference_n_layers: int = 1
@@ -141,6 +150,7 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
     inference_normalization: Literal[None, "batch", "layer"] = None
     inference_activation: str = None
     inference_dropout: float = 0.0
+    inference_regularizer: str = None
 
     # Model network parameters
     model_rnn: Literal["gru", "lstm"] = "lstm"
@@ -149,8 +159,10 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
     model_normalization: Literal[None, "batch", "layer"] = None
     model_activation: str = None
     model_dropout: float = 0.0
+    model_regularizer: str = None
 
     # Observation model parameters
+    n_fc_modes: int = None
     learn_means: bool = None
     learn_stds: bool = None
     learn_fcs: bool = None
@@ -185,6 +197,12 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         ):
             raise ValueError("learn_means, learn_stds and learn_fcs must be passed.")
 
+    def validate_dimension_parameters(self):
+        super().validate_dimension_parameters()
+        if self.n_fc_modes is None:
+            self.n_fc_modes = self.n_modes
+            print("Warning: n_fc_modes is None, set to n_modes.")
+
 
 class Model(VariationalInferenceModelBase):
     """M-DyNeMo model class.
@@ -194,6 +212,8 @@ class Model(VariationalInferenceModelBase):
     config : osl_dynamics.models.mdynemo.Config
     """
 
+    config_type = Config
+
     def build_model(self):
         """Builds a keras model."""
         self.model = _model_structure(self.config)
@@ -212,6 +232,10 @@ class Model(VariationalInferenceModelBase):
         """
         return mdynemo_obs.get_means_stds_fcs(self.model)
 
+    def get_observation_model_parameters(self):
+        """Wrapper for get_means_stds_fcs."""
+        return self.get_means_stds_fcs()
+
     def set_means_stds_fcs(self, means, stds, fcs, update_initializer=True):
         """Set the means, standard deviations, functional connectivities of each mode.
 
@@ -223,13 +247,24 @@ class Model(VariationalInferenceModelBase):
             Mode standard deviations with shape (n_modes, n_channels) or
             (n_modes, n_channels, n_channels).
         fcs: np.ndarray
-            Mode functional connectivities with shape (n_modes, n_channels, n_channels).
+            Mode functional connectivities with shape (n_fc_modes, n_channels, n_channels).
         update_initializer: bool
             Do we want to use the passed parameters when we re_initialize
             the model?
         """
         mdynemo_obs.set_means_stds_fcs(self.model, means, stds, fcs, update_initializer)
 
+    def set_observation_model_parameters(
+        self, observation_model_parameters, update_initializer=True
+    ):
+        """Wrapper for set_means_stds_fcs."""
+        self.set_means_stds_fcs(
+            observation_model_parameters[0],
+            observation_model_parameters[1],
+            observation_model_parameters[2],
+            update_initializer=update_initializer,
+        )
+
     def set_regularizers(self, training_dataset):
         """Set the regularizers of means, stds and fcs based on the training data.
 
@@ -286,7 +321,7 @@ class Model(VariationalInferenceModelBase):
             0, 1, [n_samples + 1, self.config.n_modes]
         ).astype(np.float32)
         fc_epsilon = np.random.normal(
-            0, 1, [n_samples + 1, self.config.n_modes]
+            0, 1, [n_samples + 1, self.config.n_fc_modes]
         ).astype(np.float32)
 
         # Initialise sequence of underlying logits theta
@@ -296,14 +331,14 @@ class Model(VariationalInferenceModelBase):
         )
         mean_theta_norm[-1] = np.random.normal(size=self.config.n_modes)
         fc_theta_norm = np.zeros(
-            [self.config.sequence_length, self.config.n_modes],
+            [self.config.sequence_length, self.config.n_fc_modes],
             dtype=np.float32,
         )
-        fc_theta_norm[-1] = np.random.normal(size=self.config.n_modes)
+        fc_theta_norm[-1] = np.random.normal(size=self.config.n_fc_modes)
 
         # Sample the mode time courses
         alpha = np.empty([n_samples, self.config.n_modes])
-        gamma = np.empty([n_samples, self.config.n_modes])
+        gamma = np.empty([n_samples, self.config.n_fc_modes])
         for i in trange(n_samples, desc="Sampling mode time courses", ncols=98):
             # If there are leading zeros we trim theta so that we don't pass the zeros
             trimmed_mean_theta = mean_theta_norm[~np.all(mean_theta_norm == 0, axis=1)][
@@ -358,6 +393,7 @@ def _model_structure(config):
         config.inference_n_layers,
         config.inference_n_units,
         config.inference_dropout,
+        config.inference_regularizer,
         name="inf_rnn",
     )
 
@@ -396,9 +432,9 @@ def _model_structure(config):
     #
 
     # Layers
-    fc_inf_mu_layer = layers.Dense(config.n_modes, name="fc_inf_mu")
+    fc_inf_mu_layer = layers.Dense(config.n_fc_modes, name="fc_inf_mu")
     fc_inf_sigma_layer = layers.Dense(
-        config.n_modes, activation="softplus", name="fc_inf_sigma"
+        config.n_fc_modes, activation="softplus", name="fc_inf_sigma"
     )
     fc_theta_layer = SampleNormalDistributionLayer(name="fc_theta")
     fc_theta_norm_layer = NormalizationLayer(
@@ -439,7 +475,7 @@ def _model_structure(config):
         name="stds",
     )
     fcs_layer = CorrelationMatricesLayer(
-        config.n_modes,
+        config.n_fc_modes,
         config.n_channels,
         config.learn_fcs,
         config.initial_fcs,
@@ -478,6 +514,7 @@ def _model_structure(config):
         config.model_n_layers,
         config.model_n_units,
         config.model_dropout,
+        config.model_regularizer,
         name="mod_rnn",
     )
 
@@ -509,9 +546,9 @@ def _model_structure(config):
     #
 
     # Layers
-    fc_mod_mu_layer = layers.Dense(config.n_modes, name="fc_mod_mu")
+    fc_mod_mu_layer = layers.Dense(config.n_fc_modes, name="fc_mod_mu")
     fc_mod_sigma_layer = layers.Dense(
-        config.n_modes, activation="softplus", name="fc_mod_sigma"
+        config.n_fc_modes, activation="softplus", name="fc_mod_sigma"
     )
     fc_kl_div_layer = KLDivergenceLayer(name="fc_kl_div")
 
