@@ -2,12 +2,15 @@
 
 """
 
+import os
 import re
+import pprint as pp
 from abc import abstractmethod
 from io import StringIO
 from dataclasses import dataclass
 
 import numpy as np
+import yaml
 import tensorflow
 from tensorflow.data import Dataset
 from tensorflow.keras import optimizers
@@ -16,9 +19,9 @@ from tensorflow.python.distribute.mirrored_strategy import MirroredStrategy
 from tqdm.auto import tqdm as tqdm_auto
 from tqdm.keras import TqdmCallback
 
-from osl_dynamics import data
+from osl_dynamics import data, models
 from osl_dynamics.inference import callbacks, initializers
-from osl_dynamics.utils.misc import get_argument, replace_argument
+from osl_dynamics.utils.misc import get_argument, replace_argument, NumpyLoader
 from osl_dynamics.utils.model import HTMLTable, LatexTable
 
 
@@ -27,6 +30,7 @@ class BaseModelConfig:
     """Base class for settings for all models."""
 
     # Model choices
+    model_name: str = None
     multiple_dynamics: bool = False
 
     # Training parameters
@@ -95,6 +99,8 @@ class ModelBase:
     Acts as a wrapper for a standard Keras model.
     """
 
+    config_type = None
+
     def __init__(self, config):
         self._identifier = np.random.randint(100000)
         self.config = config
@@ -183,6 +189,13 @@ class ModelBase:
                 self.model.fit, "epochs", self.config.n_epochs, args, kwargs
             )
 
+        # If we're saving the model after a particular epoch make sure it's
+        # less than the total number of epochs
+        if save_best_after is not None:
+            epochs = get_argument(self.model.fit, "epochs", args, kwargs)
+            if epochs < save_best_after:
+                raise ValueError("save_best_after must be less than epochs.")
+
         # Callbacks to add to the ones the user passed
         additional_callbacks = []
 
@@ -243,7 +256,7 @@ class ModelBase:
         self.reset_weights()
         self.compile()
 
-    def make_dataset(self, inputs, shuffle=False, concatenate=False):
+    def make_dataset(self, inputs, shuffle=False, concatenate=False, subj_id=False):
         """Converts a Data object into a TensorFlow Dataset.
 
         Parameters
@@ -255,6 +268,8 @@ class ModelBase:
             Should we shuffle the data?
         concatenate : bool
             Should we return a single TensorFlow Dataset or a list of Datasets.
+        subj_id : bool
+            Should we include the subject id in the dataset?
 
         Returns
         -------
@@ -274,6 +289,7 @@ class ModelBase:
                 self.config.batch_size,
                 shuffle=shuffle,
                 concatenate=concatenate,
+                subj_id=subj_id,
             )
         elif isinstance(inputs, Dataset) and not concatenate:
             # Dataset -> list of Dataset if concatenate=False
@@ -351,3 +367,68 @@ class ModelBase:
 
     def _repr_html_(self):
         return self.html_summary()
+
+    def save_config(self, filepath):
+        """Saves config object as a .yml file.
+
+        Parameters
+        ----------
+        filepath : str
+            Filepath to save config.yml.
+        """
+        os.makedirs(filepath, exist_ok=True)
+
+        config_dict = self.config.__dict__.copy()
+        # for serialisability of the dict
+        non_serializable_keys = [
+            key for key in list(config_dict.keys()) if "regularizer" in key
+        ]
+        non_serializable_keys.append("strategy")
+        for key in non_serializable_keys:
+            config_dict[key] = None
+
+        with open(f"{filepath}/config.yml", "w") as file:
+            yaml.dump(config_dict, file)
+
+    def save(self, filepath):
+        """Saves config object and weights of the model.
+
+        This is a wrapper for self.save_config and self.save_weights.
+
+        Parameters
+        ----------
+        filepath : str
+            Filepath to save the config object and weights of the model.
+        """
+        self.save_config(filepath)
+        self.save_weights(
+            str(filepath) + "/weights"
+        )  # will use the keras method: self.model.save_weights()
+
+    @classmethod
+    def load(cls, filepath):
+        """
+        Load model from filepath.
+
+        Parameters
+        ----------
+        filepath : str
+            Path where config.yml and weights are stored.
+
+        Returns
+        -------
+        model
+            Model object.
+        """
+
+        # Get the config
+        with open(f"{filepath}/config.yml", "r") as f:
+            config_dict = yaml.load(f, NumpyLoader)
+
+        config = cls.config_type(**config_dict)
+        model = cls(config)
+
+        # Restore weights
+        model.load_weights(str(filepath) + "/weights")
+
+        return model
