@@ -19,7 +19,7 @@ from osl_dynamics.inference.layers import (
     MixVectorsLayer,
     MixMatricesLayer,
     ModelRNNLayer,
-    MixVectorsMatricesLayer,
+    ConcatVectorsMatricesLayer,
     AdversarialLogLikelihoodLossLayer,
 )
 
@@ -30,6 +30,8 @@ class Config(BaseModelConfig):
 
     Parameters
     ----------
+    model_name : str
+        Model name.
     n_modes : int
         Number of modes.
     n_channels : int
@@ -49,6 +51,8 @@ class Config(BaseModelConfig):
         E.g. 'relu', 'elu', etc.
     inference_dropout : float
         Dropout rate.
+    inference_regularizer : str
+        Regularizer.
 
     model_rnn : str
         RNN to use, either 'gru' or 'lstm'.
@@ -63,6 +67,8 @@ class Config(BaseModelConfig):
         E.g. 'relu', 'elu', etc.
     model_dropout : float
         Dropout rate.
+    model_regularizer : str
+        Regularizer.
 
     discriminator_rnn : str
         RNN to use, either 'gru' or 'lstm'.
@@ -77,6 +83,8 @@ class Config(BaseModelConfig):
         E.g. 'relu', 'elu', etc.
     discriminator_dropout : float
         Dropout rate.
+    discriminator_regularizer : str
+        Regularizer.
 
     learn_means : bool
         Should we make the mean vectors for each mode trainable?
@@ -101,6 +109,8 @@ class Config(BaseModelConfig):
         Strategy for distributed learning.
     """
 
+    model_name: str = "SAGE"
+
     # Inference network parameters
     inference_rnn: Literal["gru", "lstm"] = "lstm"
     inference_n_layers: int = 1
@@ -108,6 +118,7 @@ class Config(BaseModelConfig):
     inference_normalization: Literal[None, "batch", "layer"] = None
     inference_activation: str = "elu"
     inference_dropout: float = 0.0
+    inference_regularizer: str = None
 
     # Model network parameters
     model_rnn: Literal["gru", "lstm"] = "lstm"
@@ -116,6 +127,7 @@ class Config(BaseModelConfig):
     model_normalization: Literal[None, "batch", "layer"] = None
     model_activation: str = "elu"
     model_dropout: float = 0.0
+    model_regularizer: str = None
 
     # Descriminator network parameters
     discriminator_rnn: Literal["gru", "lstm"] = "lstm"
@@ -124,6 +136,7 @@ class Config(BaseModelConfig):
     discriminator_normalization: Literal[None, "batch", "layer"] = None
     discriminator_activation: str = "elu"
     discriminator_dropout: float = 0.0
+    discriminator_regularizer: str = None
 
     # Observation model parameters
     learn_means: bool = None
@@ -144,6 +157,8 @@ class Model(ModelBase):
     config : osl_dynamics.models.sage.Config
     """
 
+    config_type = Config
+
     def build_model(self):
         """Builds a keras model for the inference, generator and discriminator model
         and the full SAGE model.
@@ -212,12 +227,12 @@ class Model(ModelBase):
             optimizer=optimizer,
         )
 
-    def fit(self, train_data, epochs=None, verbose=1):
+    def fit(self, training_data, epochs=None, verbose=1):
         """Train the model.
 
         Parameters
         ----------
-        train_data : tensorflow.data.Dataset
+        training_data : tensorflow.data.Dataset or osl_dynamics.data.Data
             Training dataset.
         epochs : int
             Number of epochs to train. Defaults to value in config if not passed.
@@ -232,6 +247,9 @@ class Model(ModelBase):
         if epochs is None:
             epochs = self.config.n_epochs
 
+        # Make sure training data is a TensorFlow Dataset
+        training_data = self.make_dataset(training_data, shuffle=True, concatenate=True)
+
         # Path to save the best model weights
         timestr = time.strftime("%Y%m%d-%H%M%S")  # current date-time
         filepath = "tmp/"
@@ -242,9 +260,9 @@ class Model(ModelBase):
         for epoch in range(epochs):
             if verbose:
                 print("Epoch {}/{}".format(epoch + 1, epochs))
-                pb_i = utils.Progbar(len(train_data), stateful_metrics=["D", "G"])
+                pb_i = utils.Progbar(len(training_data), stateful_metrics=["D", "G"])
 
-            for idx, batch in enumerate(train_data):
+            for idx, batch in enumerate(training_data):
                 # Generate real/fake input for the discriminator
                 real = np.ones((len(batch["data"]), self.config.sequence_length, 1))
                 fake = np.zeros((len(batch["data"]), self.config.sequence_length, 1))
@@ -292,13 +310,13 @@ class Model(ModelBase):
 
         return train
 
-    def get_alpha(self, inputs, concatenate=True):
+    def get_alpha(self, inputs, concatenate=False):
         """Mode mixing factors, alpha.
 
         Parameters
         ----------
-        inputs : tensorflow.data.Dataset
-            Prediction dataset.
+        inputs : tensorflow.data.Dataset or osl_dynamics.data.Data
+            Prediction data.
 
         Returns
         -------
@@ -306,6 +324,9 @@ class Model(ModelBase):
             Mode mixing factors with shape (n_subjects, n_samples, n_modes) or
             (n_samples, n_modes).
         """
+        inputs = self.make_dataset(inputs, concatenate=concatenate)
+
+        print("Getting alpha:")
         outputs = []
         for dataset in inputs:
             alpha = self.inference_model.predict(dataset)[1]
@@ -418,6 +439,7 @@ def _build_inference_model(config):
         config.inference_n_layers,
         config.inference_n_units,
         config.inference_dropout,
+        config.inference_regularizer,
         name="inf_rnn",
     )
 
@@ -453,14 +475,14 @@ def _build_inference_model(config):
     )
     mix_means_layer = MixVectorsLayer(name="mix_means")
     mix_covs_layer = MixMatricesLayer(name="mix_covs")
-    mix_means_covs_layer = MixVectorsMatricesLayer(name="mix_means_covs")
+    concat_means_covs_layer = ConcatVectorsMatricesLayer(name="concat_means_covs")
 
     # Data flow
     mu = means_layer(inputs)  # inputs not used
     D = covs_layer(inputs)  # inputs not used
     m = mix_means_layer([alpha, mu])
     C = mix_covs_layer([alpha, D])
-    C_m = mix_means_covs_layer([m, C])
+    C_m = concat_means_covs_layer([m, C])
 
     return models.Model(inputs, [C_m, alpha], name="inference")
 
@@ -486,6 +508,7 @@ def _build_generator_model(config):
         config.model_n_layers,
         config.model_n_units,
         config.model_dropout,
+        config.model_regularizer,
         name="gen_rnn",
     )
     prior_layer = layers.TimeDistributed(
@@ -517,6 +540,7 @@ def _build_discriminator_model(config):
         config.discriminator_n_layers,
         config.discriminator_n_units,
         config.discriminator_dropout,
+        config.discriminator_regularizer,
         name="dis_rnn",
     )
     sigmoid_layer = layers.TimeDistributed(
