@@ -41,15 +41,20 @@ from osl_dynamics.inference.layers import (
 
 @dataclass
 class Config(BaseModelConfig, VariationalInferenceModelConfig):
-    """Settings for SERIGO.
+    """Settings for SE-DyNeMo.
+
     Parameters
     ----------
+    model_name : str
+        Model name.
+
     n_modes : int
         Number of modes.
     n_channels : int
         Number of channels.
     sequence_length : int
         Length of sequence passed to the inference network and generative model.
+
     inference_rnn : str
         RNN to use, either 'gru' or 'lstm'.
     inference_n_layers : int
@@ -63,6 +68,9 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         E.g. 'relu', 'elu', etc.
     inference_dropout : float
         Dropout rate.
+    inference_regularizer : str
+        Regularizer.
+
     model_rnn : str
         RNN to use, either 'gru' or 'lstm'.
     model_n_layers : int
@@ -76,6 +84,9 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         E.g. 'relu', 'elu', etc.
     model_dropout : float
         Dropout rate.
+    model_regularizer : str
+        Regularizer.
+
     theta_normalization : str
         Type of normalization to apply to the posterior samples, theta.
         Either 'layer', 'batch' or None.
@@ -143,6 +154,8 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         Initial value for prior std of the deviation.
     """
 
+    model_name: str = "SE-DyNeMo"
+
     # Inference network parameters
     inference_rnn: Literal["gru", "lstm"] = "lstm"
     inference_n_layers: int = 1
@@ -150,6 +163,7 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
     inference_normalization: Literal[None, "batch", "layer"] = None
     inference_activation: str = None
     inference_dropout: float = 0.0
+    inference_regularizer: str = None
 
     # Model network parameters
     model_rnn: Literal["gru", "lstm"] = "lstm"
@@ -158,9 +172,9 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
     model_normalization: Literal[None, "batch", "layer"] = None
     model_activation: str = None
     model_dropout: float = 0.0
+    model_regularizer: str = None
 
     # Observation model parameters
-    multiple_scales: bool = False
     learn_means: bool = None
     learn_covariances: bool = None
     initial_means: np.ndarray = None
@@ -217,10 +231,16 @@ class Model(VariationalInferenceModelBase):
     config : dynemo.models.serigo.Config
     """
 
+    config_type = Config
+
     def build_model(self):
         """Builds a keras model."""
         self.model = _model_structure(self.config)
 
+    def make_dataset(self, inputs, shuffle=False, concatenate=False, subj_id=True):
+        """SE-DyNeMo requires subject id to be included in the dataset."""
+        return super().make_dataset(inputs, shuffle, concatenate, subj_id)
+
     def get_group_means_covariances(self):
         """Get the group means and covariances of each mode
         Returns
@@ -232,6 +252,10 @@ class Model(VariationalInferenceModelBase):
         """
         return sedynemo_obs.get_group_means_covariances(self.model)
 
+    def get_observation_model_parameters(self):
+        """Wrapper for get_group_means_covariances."""
+        return self.get_group_means_covariances()
+
     def get_subject_embeddings(self):
         """Get the subject embedding vectors
         Returns
@@ -297,7 +321,7 @@ class Model(VariationalInferenceModelBase):
 
         A multivariate normal prior is applied to the mean vectors with mu = 0,
         sigma=diag((range / 2)**2) and an inverse Wishart prior is applied to the
-        covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(1 / range).
+        covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(range).
 
         Parameters
         ----------
@@ -316,8 +340,57 @@ class Model(VariationalInferenceModelBase):
                 self.model, training_dataset, layer_name="group_covs"
             )
 
+    def set_group_means(self, group_means, update_initializer=True):
+        """Set the group means of each mode.
+
+        Parameters
+        ----------
+        group_means : np.ndarray
+            Mode means.
+        update_initializer : bool
+            Do we want to use the passed group means when we re-initialize the model?
+        """
+        dynemo_obs.set_means(
+            self.model, group_means, update_initializer, layer_name="group_means"
+        )
+
+    def set_group_covariances(self, group_covariances, update_initializer=True):
+        """Set the group covariances of each mode.
+
+        Parameters
+        ----------
+        group_covariances : np.ndarray
+            Mode covariances.
+        update_initializer : bool
+            Do we want to use the passed group covariances when we re-initialize
+            the model?
+        """
+        dynemo_obs.set_covariances(
+            self.model, group_covariances, update_initializer, layer_name="group_covs"
+        )
+
+    def set_observation_model_parameters(
+        self, observation_model_parameters, update_initializer=True
+    ):
+        """Wrapper for set_group_means and set_group_covariances."""
+        self.set_group_means(
+            observation_model_parameters[0],
+            update_initializer=update_initializer,
+        )
+        self.set_group_covariances(
+            observation_model_parameters[1],
+            update_initializer=update_initializer,
+        )
+
     def set_bayesian_kl_scaling(self, training_dataset):
-        """Set the correct scaling for KL loss between deviation posterior and prior."""
+        """Set the correct scaling for KL loss between deviation posterior and prior.
+
+        Parameters
+        ----------
+        training_dataset : tensorflow.data.Dataset or osl_dynamics.data.Data
+            Training dataset.
+        """
+        training_dataset = self.make_dataset(training_dataset, concatenate=True)
         n_batches = dtf.get_n_batches(training_dataset)
         learn_means = self.config.learn_means
         learn_covariances = self.config.learn_covariances
@@ -325,6 +398,14 @@ class Model(VariationalInferenceModelBase):
             self.model, n_batches, learn_means, learn_covariances
         )
 
+    def random_subject_initialization(
+        self, training_data, n_epochs, n_subjects, n_kl_annealing_epochs=None, **kwargs
+    ):
+        """random subject initialisation not compatible with SE-DyNeMo."""
+        raise AttributeError(
+            " 'Model' object has no attribute 'random_subject_initialization'."
+        )
+
 
 def _model_structure(config):
     # layers for inputs
@@ -347,6 +428,7 @@ def _model_structure(config):
         config.inference_n_layers,
         config.inference_n_units,
         config.inference_dropout,
+        config.inference_regularizer,
         name="inf_rnn",
     )
     inf_mu_layer = layers.Dense(config.n_modes, name="inf_mu")
@@ -566,6 +648,7 @@ def _model_structure(config):
         config.model_n_layers,
         config.model_n_units,
         config.model_dropout,
+        config.model_regularizer,
         name="mod_rnn",
     )
     concatenate_layer = ConcatenateLayer(axis=2, name="model_concat")
